/* Contains the following:
     init.c
     start.c
     stop.c
     read.c
     write.c
*/

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>

#define LPT_READ  0x379
#define LPT_WRITE 0x37A



void iic_initialise(void)
{
   /* Sets the initial state, and checks it... */
   outp( LPT_WRITE, 0 );

   if ( (inp(LPT_READ) & 0xC0) != 0x40 )
   {
      printf("IIC bus hardware error!\n\n");
      exit(EXIT_FAILURE);
   }
   return;
}



void iic_start(void)
{
   /* Sets up a start condition. */

   if ( ( inp(LPT_READ) & 0x40 ) == 0 )
   {
      /* Clock is already low; so set it high.  */
      outp(LPT_WRITE, 2); /* SDA high, SCL low  */
      outp(LPT_WRITE, 0); /* SDA high, SCL high */
   }

   outp(LPT_WRITE, 1);    /* SDA low, SCL high  */
   outp(LPT_WRITE, 3);    /* SDA low, SCL low   */

   return;
}



void iic_stop(void)
{
   /* Send a stop condition. */

   outp(LPT_WRITE, 3);    /* SDA low, SCL low   */
   outp(LPT_WRITE, 1);    /* SDA low, SCL high  */
   outp(LPT_WRITE, 0);    /* SDA high, SCL high */

   return;
}



int  iic_read(int doack)
{
   /* Reads a byte off the IIC bus...
      ASSUMES WE'VE STARTED.

      WE, THE 'MASTER', CLOCK THE DATA IN, SO WE
      KNOW THE 'SLAVE' IS RUNNING TO OUR SPEED.
   */
   int  bit_count = 0;
   int  this_byte = 0;

   for (bit_count = 7; bit_count >= 0; bit_count--)
   {
      outp(LPT_WRITE, 2);
            /* SCL low, SDA high;
               allows slave to pull SDA */
      outp(LPT_WRITE, 0);
            /* SCL high, SDA now = bit from slave */

      /* clocked it in, so read it... */
      this_byte = this_byte * 2;
      this_byte = this_byte + \
                  (1 -((inp(LPT_READ) & 128) >> 7));

      outp(LPT_WRITE, 2);
            /* SCL low, SDA = bit from slave */
   }

   /* Ack it (remember the signal inversions) */
   outp(LPT_WRITE, (2 + doack));
            /* SCL low, SDA = (not)DoAck */
   outp(LPT_WRITE, doack);
            /* SCL high, SDA = (not)DoAck */
   outp(LPT_WRITE, (2 + doack));
            /* SCL low, SDA = (not)DoAck */
   outp(LPT_WRITE, 3);
            /* SCL low, SDA low */

   return this_byte;
}

void iic_write(int value)
{
   /* Writes a byte to the IIC bus...
      ASSUMES WE'VE STARTED.          */

   int  bit_count = 0;
   int  bit  = 0;
   int  ack = 0;

   for (bit_count = 7; bit_count >= 0; bit_count--)
   {
      this_bit = ((value & 128) >> 7);
      outp(LPT_WRITE, ((1 - this_bit) + 2) );
                          /* SDA = bit, SCL low  */
      outp(LPT_WRITE, (1 - this_bit));
                          /* SDA = bit, SCL high */
      value = ((value & 127) * 2);
                          /* Set up for next bit */
      outp(LPT_WRITE, ((1 - this_bit) + 2) );
                          /* SDA = bit, SCL low  */
   }

   /* Fake ninth bit, for Ack */
   outp(LPT_WRITE, 2);    /* SDA high, SCL low   */
   outp(LPT_WRITE, 0);    /* SDA high, SCL low   */
   ack = 1-((inp(LPT_READ) & 128) >> 7);
   /* Tidy up, but we dont clock it */
   outp(LPT_WRITE, 2);    /* SDA high, SCL low   */
   outp(LPT_WRITE, 3);    /* SDA low, SCL low    */

   if (ack != 0)
   {
      printf("No acknowledge from IIC device.\n");
      /* Clear IIC bus */
      outp(LPT_WRITE, 0);
      exit(EXIT_FAILURE);
   }
   return;
}
